@@ -48,11 +48,17 @@ from viewer import PitiviViewer
 from pitivi.bin import SmartTimelineBin
 from projectsettings import ProjectSettingsDialog
 from pluginmanagerdialog import PluginManagerDialog
+from webcam_managerdialog import WebcamManagerDialog
+from netstream_managerdialog import NetstreamManagerDialog
+from screencast_managerdialog import ScreencastManagerDialog
 from pitivi.configure import pitivi_version, APPNAME
 from glade import GladeWindow
-
 from exportsettingswidget import ExportSettingsDialog
 
+import dbus
+import dbus.service
+import dbus.glib
+
 if have_gconf:
     D_G_INTERFACE = "/desktop/gnome/interface"
 
@@ -90,6 +96,21 @@ class PitiviMainWindow(gtk.Window):
         instance.PiTiVi.playground.connect("error", self._playGroundErrorCb)
         instance.PiTiVi.current.sources.connect_after("file_added", self._sourcesFileAddedCb)
 
+        # Start dbus service
+        session_bus = dbus.SessionBus()
+        name = dbus.service.BusName("org.gnome.pitivi", bus=session_bus)
+        object = pitivi_dbus(name)
+
+
+        # FIXME : connect to the device probe signals to toggle the webcam
+        # features based on availability (or not) of webcams.
+
+        # if no webcams available, hide the webcam action
+        instance.PiTiVi.deviceprobe.connect("device-added", self.__deviceChangeCb)
+        instance.PiTiVi.deviceprobe.connect("device-removed", self.__deviceChangeCb)
+        if len(instance.PiTiVi.deviceprobe.getVideoSourceDevices()) < 1:
+            self.webcam_button.set_sensitive(False)
+
         self.show_all()
 
     def _encodingDialogDestroyCb(self, unused_dialog):
@@ -109,12 +130,10 @@ class PitiviMainWindow(gtk.Window):
         if not isinstance(instance.PiTiVi.playground.current, SmartTimelineBin):
             return
         self.render_button.set_sensitive((duration > 0) and True or False)
-        if duration > 0 :
-            gobject.idle_add(self.timeline.simpleview._displayTimeline)
 
     def _currentPlaygroundChangedCb(self, playground, smartbin):
-	if smartbin == playground.default:
-	    self.render_button.set_sensitive(False)
+        if smartbin == playground.default:
+            self.render_button.set_sensitive(False)
         else:
             if isinstance(smartbin, SmartTimelineBin):
                 gst.info("switching to Timeline, setting duration to %s" %
@@ -122,24 +141,18 @@ class PitiviMainWindow(gtk.Window):
                 smartbin.project.timeline.videocomp.connect("start-duration-changed",
                                                             self._timelineDurationChangedCb)
                 if smartbin.project.timeline.videocomp.duration > 0:
-		    self.render_button.set_sensitive(True)
-                    gobject.idle_add(self.timeline.simpleview._displayTimeline)
+                    self.render_button.set_sensitive(True)
                 else:
-		    self.render_button.set_sensitive(False)
+                    self.render_button.set_sensitive(False)
             else:
-		self.render_button.set_sensitive(False)
+                self.render_button.set_sensitive(False)
 
     def _createStockIcons(self):
         """ Creates the pitivi-only stock icons """
         gtk.stock_add([
-                ('pitivi-advanced-mode', 'Advanced Mode', 0, 0, 'pitivi'),
                 ('pitivi-render', 'Render', 0, 0, 'pitivi')
                 ])
         factory = gtk.IconFactory()
-        pixbuf = gtk.gdk.pixbuf_new_from_file(
-            configure.get_pixmap_dir() + "/pitivi-advanced-24.png")
-        iconset = gtk.IconSet(pixbuf)
-        factory.add('pitivi-advanced-mode', iconset)
         pixbuf = gtk.gdk.pixbuf_new_from_file(
             configure.get_pixmap_dir() + "/pitivi-render-24.png")
         iconset = gtk.IconSet(pixbuf)
@@ -170,18 +183,25 @@ class PitiviMainWindow(gtk.Window):
             ("PluginManager", gtk.STOCK_PREFERENCES ,
              _("_Plugins..."),
              None, _("Manage plugins"), self._pluginManagerCb),
+            ("ImportfromCam", gtk.STOCK_ADD ,
+             _("_Import from Webcam.."),
+             None, _("Import Camera stream"), self._ImportWebcam),   
+            ("Screencast", gtk.STOCK_ADD ,
+             _("_Make screencast.."),
+             None, _("Capture the desktop"), self._Screencast),   
+            ("NetstreamCapture", gtk.STOCK_ADD ,
+             _("_Capture Network Stream.."),
+             None, _("Capture Network Stream"), self._ImportNetstream), 
             ("Quit", gtk.STOCK_QUIT, None, None, None, self._quitCb),
             ("About", gtk.STOCK_ABOUT, None, None,
              _("Information about %s") % APPNAME, self._aboutCb),
             ("File", None, _("_File")),
             ("Edit", None, _("_Edit")),
             ("View", None, _("_View")),
-            ("Help", None, _("_Help"))
+            ("Help", None, _("_Help")),
         ]
 
         self.toggleactions = [
-            ("AdvancedView", 'pitivi-advanced-mode', _("Advanced vie_w"),
-             None, _("Switch to advanced view"), self._advancedViewCb),
             ("FullScreen", gtk.STOCK_FULLSCREEN, None, None,
              _("View the main window on the whole screen"), self._fullScreenCb)
         ]
@@ -195,13 +215,16 @@ class PitiviMainWindow(gtk.Window):
         for action in self.actiongroup.list_actions():
             if action.get_name() == "RenderProject":
                 self.render_button = action
-            elif action.get_name() == "AdvancedView":
-                if not instance.PiTiVi.settings.advancedModeEnabled:
-                    action.set_visible(False)
+            elif action.get_name() == "ImportfromCam":
+                self.webcam_button = action
+            elif action.get_name() == "Screencast":
+                # FIXME : re-enable this action once istanbul integration is complete
+                # and upstream istanbul has applied packages for proper interaction.
+                action.set_visible(False)
             elif action.get_name() in [
                 "ProjectSettings", "Quit", "File", "Edit", "Help",
                 "About", "View", "FullScreen", "ImportSources",
-                "ImportSourcesFolder", "AdvancedView", "PluginManager"]:
+                "ImportSourcesFolder", "PluginManager","ImportfromCam","NetstreamCapture"]:
                 action.set_sensitive(True)
             elif action.get_name() in ["SaveProject", "SaveProjectAs",
                     "NewProject", "OpenProject"]:
@@ -210,17 +233,18 @@ class PitiviMainWindow(gtk.Window):
             else:
                 action.set_sensitive(False)
 
-        self.uimanager = gtk.UIManager()
+        self.uimanager = instance.PiTiVi.uimanager
         self.add_accel_group(self.uimanager.get_accel_group())
         self.uimanager.insert_action_group(self.actiongroup, 0)
-        self.uimanager.add_ui_from_file(os.path.join(os.path.dirname(os.path.abspath(__file__)), "actions.xml"))
+        self.uimanager.add_ui_from_file(os.path.join(os.path.dirname(
+            os.path.abspath(__file__)), "actions.xml"))
 
         self.connect_after("key-press-event", self._keyPressEventCb)
 
     def _createUi(self):
         """ Create the graphical interface """
         self.set_title("%s v%s" % (APPNAME, pitivi_version))
-        self.set_geometry_hints(min_width=800, min_height=600)
+        self.set_geometry_hints(min_width=800, min_height=480)
 
         self.connect("destroy", self._destroyCb)
 
@@ -239,10 +263,9 @@ class PitiviMainWindow(gtk.Window):
         vbox.pack_start(vpaned)
 
         self.timeline = TimelineWidget()
-        self.timeline.showSimpleView()
-        timelineframe = gtk.Frame()
-        timelineframe.add(self.timeline)
-        vpaned.pack2(timelineframe, resize=False, shrink=False)
+        self.timeline.showComplexView()
+
+        vpaned.pack2(self.timeline, resize=False, shrink=True)
 
         hpaned = gtk.HPaned()
         vpaned.pack1(hpaned, resize=True, shrink=False)
@@ -253,13 +276,20 @@ class PitiviMainWindow(gtk.Window):
         # Viewer
         self.viewer = PitiviViewer()
 
-	instance.PiTiVi.playground.connect("current-changed", self._currentPlaygroundChangedCb)
+        instance.PiTiVi.playground.connect("current-changed", 
+            self._currentPlaygroundChangedCb)
 
         hpaned.pack1(self.sourcefactories, resize=False, shrink=False)
         hpaned.pack2(self.viewer, resize=True, shrink=False)
+        # FIXME: remove toolbar padding and shadow. In fullscreen mode, the
+        # toolbar buttons should be clickable with the mouse cursor at the
+        # very bottom of the screen.
+        vbox.pack_start(self.uimanager.get_widget("/TimelineToolBar"),
+            False)
 
         #application icon
-        self.set_icon_from_file(configure.get_global_pixmap_dir() + "/pitivi.png")
+        self.set_icon_from_file(configure.get_global_pixmap_dir() 
+            + "/pitivi.png")
 
     def toggleFullScreen(self):
         """ Toggle the fullscreen mode of the application """
@@ -270,7 +300,7 @@ class PitiviMainWindow(gtk.Window):
             self.viewer.window.unfullscreen()
             self.isFullScreen = False
 
-    ## PlayGround callback
+## PlayGround callback
 
     def _errorMessageResponseCb(self, dialogbox, unused_response):
         dialogbox.hide()
@@ -278,29 +308,44 @@ class PitiviMainWindow(gtk.Window):
         self.errorDialogBox = None
 
     def _playGroundErrorCb(self, unused_playground, error, detail):
+        # FIXME FIXME FIXME:
+        # _need_ an onobtrusive way to present gstreamer errors, 
+        # one that doesn't steel mouse/keyboard focus, one that
+        # makes some kind of sense to the user, and one that presents
+        # some ways of actually _dealing_ with the underlying problem:
+        # install a plugin, re-conform source to some other format, or
+        # maybe even disable playback of a problematic file.
         if self.errorDialogBox:
             return
         self.errorDialogBox = gtk.MessageDialog(None, gtk.DIALOG_MODAL,
-                                                gtk.MESSAGE_ERROR,
-                                                gtk.BUTTONS_OK,
-                                                None)
+            gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, None)
         self.errorDialogBox.set_markup("<b>%s</b>" % error)
         self.errorDialogBox.connect("response", self._errorMessageResponseCb)
         if detail:
             self.errorDialogBox.format_secondary_text(detail)
         self.errorDialogBox.show()
 
-
-    ## Project source list callbacks
+## Project source list callbacks
 
     def _sourcesFileAddedCb(self, unused_sources, unused_factory):
-        if len(self.sourcefactories.sourcelist.storemodel) == 1 and not len(instance.PiTiVi.current.timeline.videocomp):
-            self.timeline.simpleview._displayTimeline(False)
+        #if (len(self.sourcefactories.sourcelist.storemodel) == 1 
+        #    and not len(instance.PiTiVi.current.timeline.videocomp):
+        pass
 
-
-    ## UI Callbacks
+## UI Callbacks
 
     def _destroyCb(self, unused_widget, data=None):
+
+        # Quit connection to istanbul/switch off savemode
+        try:
+            bus = dbus.SessionBus()
+            remote_object = bus.get_object("org.gnome.istanbul", "/state")
+            self.iface = dbus.Interface(remote_object, "org.gnome.istanbul")
+            self.iface.savemode(False)
+        except:
+            pass
+
+
         instance.PiTiVi.shutdown()
 
 
@@ -308,7 +353,7 @@ class PitiviMainWindow(gtk.Window):
         if gtk.gdk.keyval_name(event.keyval) in ['f', 'F', 'F11']:
             self.toggleFullScreen()
 
-    ## Toolbar/Menu actions callback
+## Toolbar/Menu actions callback
 
     def _newProjectMenuCb(self, unused_action):
         instance.PiTiVi.newBlankProject()
@@ -356,12 +401,6 @@ class PitiviMainWindow(gtk.Window):
     def _fullScreenCb(self, unused_action):
         self.toggleFullScreen()
 
-    def _advancedViewCb(self, action):
-        if action.get_active():
-            self.timeline.showComplexView()
-        else:
-            self.timeline.showSimpleView()
-
     def _aboutResponseCb(self, dialog, unused_response):
         dialog.destroy()
 
@@ -380,7 +419,8 @@ class PitiviMainWindow(gtk.Window):
                    "Johan Dahlin <jdahlin@async.com.br> (UI)",
                    "Brandon Lewis <brandon_lewis@berkeley.edu> (UI)",
                    "Luca Della Santina <dellasantina@farm.unipi.it>",
-                   "Thijs Vermeir <thijsvermeir@gmail.com>"]
+                   "Thijs Vermeir <thijsvermeir@gmail.com>",
+                   "Sarath Lakshman <sarathlakshman@slynux.org>"]
         abt.set_authors(authors)
         abt.set_license(_("GNU Lesser General Public License\nSee http://www.gnu.org/copyleft/lesser.html for more details"))
         abt.set_icon_from_file(configure.get_global_pixmap_dir() + "/pitivi.png")
@@ -396,6 +436,29 @@ class PitiviMainWindow(gtk.Window):
     def _pluginManagerCb(self, unused_action):
         PluginManagerDialog(instance.PiTiVi.plugin_manager)
 
+
+
+    # Import from Webcam callback
+    def _ImportWebcam(self,unused_action):
+        w = WebcamManagerDialog(instance.PiTiVi)
+        w.show()
+
+    # Capture network stream callback
+    def _ImportNetstream(self,unused_action):
+        NetstreamManagerDialog()
+
+    # screencast callback
+    def _Screencast(self,unused_action):
+        ScreencastManagerDialog()
+
+
+    ## Devices changed
+    def __deviceChangeCb(self, probe, unused_device):
+        if len(probe.getVideoSourceDevices()) < 1:
+            self.webcam_button.set_sensitive(False)
+        else:
+            self.webcam_button.set_sensitive(True)
+
     ## PiTiVi main object callbacks
 
     def _newProjectLoadedCb(self, pitivi, project):
@@ -437,7 +500,7 @@ class PitiviMainWindow(gtk.Window):
         dialog.destroy()
         self.set_sensitive(True)
 
-    ## PiTiVi current project callbacks
+## PiTiVi current project callbacks
 
     def _confirmOverwriteCb(self, unused_project, uri):
         message = _("Do you wish to overwrite existing file \"%s\"?") %\
@@ -457,6 +520,7 @@ class PitiviMainWindow(gtk.Window):
         return False
 
     def _saveAsDialogCb(self, project):
+        gst.log("Save URI requested")
         chooser = gtk.FileChooserDialog(_("Save As..."),
             self,
             action=gtk.FILE_CHOOSER_ACTION_SAVE,
@@ -465,7 +529,6 @@ class PitiviMainWindow(gtk.Window):
 
         chooser.set_select_multiple(False)
         chooser.set_current_name(_("Untitled.pptv"))
-        chooser.set_default_response(gtk.RESPONSE_CANCEL)
         formats = ProjectSaver.listFormats()
         default = gtk.FileFilter()
         default.set_name(_("Detect Automatically"))
@@ -481,18 +544,34 @@ class PitiviMainWindow(gtk.Window):
         response = chooser.run()
 
         if response == gtk.RESPONSE_OK:
+            gst.log("User chose a URI to save project to")
             # need to do this to work around bug in gst.uri_construct
             # which escapes all /'s in path!
             uri = "file://" + chooser.get_filename()
             format = chooser.get_filter().get_name()
             if format == _("Detect Automatically"):
                 format = None
+            gst.log("uri:%s , format:%s" % (uri, format))
             project.setUri(uri, format)
-            chooser.destroy()
-            return True
-        chooser.destroy()
-        return False
+            ret = True
+        else:
+            gst.log("User didn't choose a URI to save project to")
+            ret = False
 
+        chooser.destroy()
+        return ret
+
+
+# dbus object class
+class pitivi_dbus(dbus.service.Object):
+    def __init__(self, bus_name, object_path="/import"):
+        dbus.service.Object.__init__(self, bus_name, object_path)
+        self.sourcefactories = SourceFactoriesWidget()
+
+    @dbus.service.method("org.gnome.pitivi")
+    def AddFiles(self, filepath):
+        self.sourcefactories.sourcelist.addFiles(filepath)
+        return True
 
 class EncodingDialog(GladeWindow):
     """ Encoding dialog box """
@@ -507,6 +586,8 @@ class EncodingDialog(GladeWindow):
         self.eosid = self.bus.connect("message::eos", self._eosCb)
         self.outfile = None
         self.progressbar = self.widgets["progressbar"]
+        self.filebutton = self.widgets["filebutton"]
+        self.settingsbutton = self.widgets["settingsbutton"]
         self.cancelbutton = self.widgets["cancelbutton"]
         self.recordbutton = self.widgets["recordbutton"]
         self.recordbutton.set_sensitive(False)
@@ -560,6 +641,8 @@ class EncodingDialog(GladeWindow):
                 self.cancelbutton.set_label("gtk-cancel")
                 self.progressbar.set_text(_("Rendering"))
                 self.recordbutton.set_sensitive(False)
+                self.filebutton.set_sensitive(False)
+                self.settingsbutton.set_sensitive(False)
             else:
                 self.progressbar.set_text(_("Couldn't start rendering"))
 
@@ -585,6 +668,8 @@ class EncodingDialog(GladeWindow):
         self.progressbar.set_text(_("Rendering Complete"))
         self.progressbar.set_fraction(1.0)
         self.recordbutton.set_sensitive(True)
+        self.filebutton.set_sensitive(True)
+        self.settingsbutton.set_sensitive(True)
         self.cancelbutton.set_label("gtk-close")
 
     def _cancelButtonClickedCb(self, unused_button):
